import { expect } from 'chai';
import { ConsoleLogger } from 'noicejs';

import { parseArgs } from '../../src/args.js';
import { buildCache, Cache, memoryDelete, memoryGet, memoryList, memorySet } from '../../src/storage/memory.js';
import { buildLock } from '../../src/utils/lock.js';
import { REQUIRED_TEST_ARGS } from '../helpers.js';

describe('in-memory storage', () => {
  it('should list all locks in the cache', async () => {
    const args = await parseArgs(REQUIRED_TEST_ARGS);
    const logger = ConsoleLogger.global;
    const cache: Cache = new Map();

    const locks = await memoryList({ args, logger }, cache);
    expect(locks.length).to.equal(cache.size);
  });

  it('should remove locks from the cache', async () => {
    const args = await parseArgs(REQUIRED_TEST_ARGS);
    const logger = ConsoleLogger.global;
    const cache: Cache = new Map();

    await memoryDelete({ args, logger }, cache, 'test');
    expect(cache.size).to.equal(0);
  });

  it('should get locks from the cache', async () => {
    const args = await parseArgs(REQUIRED_TEST_ARGS);
    const logger = ConsoleLogger.global;
    const cache: Cache = new Map();

    const lock = await memoryGet({ args, logger }, cache, 'test');
    expect(lock).to.not.equal(undefined);
  });

  it('should add a lock to the cache', async () => {
    const args = await parseArgs([
      ...REQUIRED_TEST_ARGS,
      '--duration=5m',
    ]);
    const logger = ConsoleLogger.global;
    const cache: Cache = new Map();

    await memorySet({ args, logger }, cache, buildLock(args));
    expect(cache.size).to.equal(1);
  });

  it('should prime the cache with fake locks', async () => {
    const args = await parseArgs([
      ...REQUIRED_TEST_ARGS,
      '--fake=test:{}',
    ]);
    const logger = ConsoleLogger.global;

    const cache = buildCache({ args, logger });
    expect(cache.size).to.equal(1);
  });
});
